from gettext import gettext as _

import gtk
import gedit
from xmlprocessor import XMLToolsXmlProcessor

# Insert a new item in the Tools menu
ui_str = """<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsOps_2">
        <menuitem name="XMLTools" action="XMLvalidate"/>
        <menuitem name="XMLTools1" action="XMLxpath"/>
        <menuitem name="XMLTools2" action="XMLxquery"/>
      </placeholder>
    </menu>
  </menubar>
</ui>
"""
class XMLToolsWindowHelper:
	def __init__(self, plugin, window):
		self._window = window
		self._plugin = plugin
		
		# add bottom panel field
		self._panel_field = gtk.TextView()
		self._output_buffer = gtk.TextBuffer()
		
		self._panel_field.set_buffer(self._output_buffer)
		
		panel = window.get_bottom_panel()
		panel.add_item(self._panel_field, "XML Tools", "gtk-properties")

		# Insert menu items
		self._insert_menu()

	def deactivate(self):
		# Remove any installed menu items
		self._remove_menu()

		self._window = None
		self._plugin = None
		self._action_group = None
		
		panel = window.get_bottom_panel()
		panel.remove_item(self._panel_field)
		self._panel_field = None

	def _insert_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()

		# Create a new action group
		self._action_group = gtk.ActionGroup("XMLToolsPyPluginActions")
		self._action_group.add_actions([("XMLvalidate", None, _("XML validation"),
										'F5', _("validate an XML file"),
										self.validate_document)])
		self._action_group.add_actions([("XMLxpath", None, _("XPath query..."),
										'F6', _("XPath query editor"),
										self.xpath_query_on_document)])
		self._action_group.add_actions([("XMLxquery", None, _("XQuery..."),
										'F7', _("XQuery editor"),
										self.xquery_on_document)])

		# Insert the action group
		manager.insert_action_group(self._action_group, -1)

		# Merge the UI
		self._ui_id = manager.add_ui_from_string(ui_str)

	def _remove_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()

		# Remove the ui
		manager.remove_ui(self._ui_id)

		# Remove the action group
		manager.remove_action_group(self._action_group)

		# Make sure the manager updates
		manager.ensure_update()

	def update_ui(self):
		self._action_group.set_sensitive(self._window.get_active_document() != None)

	# Menu activate handlers
	def validate_document(self, action):
		doc = self._window.get_active_document()
		if not doc:
			return
		
		uri = doc.get_uri_for_display() 
		buff = "Validating: " + uri + "\n"
		xml = XMLToolsXmlProcessor(uri)
		if xml.valid:
			buff += "Document is valid!"
		else:
			buff += "Document is NOT valid!\n"
			for t in xml.get_error():
				buff += "Error on line: " + str(t[0]) + " -- " + t[1] + "\n"
		
		self._output_buffer.set_text(buff)
		panel = self._window.get_bottom_panel()
		panel.activate_item(self._panel_field)
		panel.set_property("visible", True)
		

	def xpath_query_on_document(self, action):
		doc = self._window.get_active_document()
		if not doc:
			return

		print "Xpath query!"

	def xquery_on_document(self, action):
		doc = self._window.get_active_document()
		if not doc:
			return

		print "XQuery!"


class XMLTools(gedit.Plugin):
	def __init__(self):
		gedit.Plugin.__init__(self)
		self._instances = {}

	def activate(self, window):
		self._instances[window] = XMLToolsWindowHelper(self, window)

	def deactivate(self, window):
		self._instances[window].deactivate()
		del self._instances[window]

	def update_ui(self, window):
		self._instances[window].update_ui()

